﻿using System;
using System.ComponentModel;
using System.Data.Services.Client;
using System.Linq;
using System.Threading;
using GameEditor.Client.Data.DataServices;
using GameEditor.Client.Data.EventArgs;
using GameEditor.Common;
using Microsoft.Practices.Composite.Logging;
using System.Windows;

namespace GameEditor.Client.Data
{
  public class GameModel : IGameModel
  {
    // Fields
    GameEntities ctx = new GameEntities(new Uri("GameService.svc", UriKind.Relative));
    ILoggerFacade theLogger;

    // Construction
    public GameModel(ILoggerFacade logger)
    {
      theLogger = logger;
    }

    // Events
    public event EventHandler<LoadEntityCompleteEventArgs<Game>> LoadGamesComplete;
    public event EventHandler<LoadEntityCompleteEventArgs<Genre>> LoadGenresComplete;
    public event EventHandler<LoadEntityCompleteEventArgs<Rating>> LoadRatingsComplete;
    public event EventHandler<ErrorEventArgs> SaveComplete;
    
    // Operations
    public void LoadGamesAsync()
    {
      var qry = (from g in ctx.Games
                 orderby g.ReleaseDate descending
                 select g) as DataServiceQuery<Game>;

      qry.BeginExecute(new AsyncCallback(OnLoadGamesComplete), qry);
    }

    public void LoadGamesByGenreAsync(string genre)
    {
      var qry = (from g in ctx.Games
                 where g.Genre.Name == genre
                 orderby g.ReleaseDate descending
                 select g) as DataServiceQuery<Game>;

      qry.BeginExecute(new AsyncCallback(OnLoadGamesComplete), qry);
    }

    public void LoadGenresAsync()
    {
      var qry = (from g in ctx.Genres
                 orderby g.Name
                 select g) as DataServiceQuery<Genre>;

      qry.BeginExecute(new AsyncCallback(OnGenresComplete), qry);
    }

    public void LoadRatingsAsync()
    {
      var qry = (from g in ctx.Ratings
                 orderby g.Name
                 select g) as DataServiceQuery<Rating>;

      qry.BeginExecute(new AsyncCallback(OnRatingsComplete), qry);
    }

    // NOTE: This saves everything that's changed in the Model, 
    //       not just the currently edited item. 
    //       Could change this behavior and support undo and such
    //       but not relevant to the way that Prism works
    public void SaveAsync()
    {
      ctx.BeginSaveChanges(SaveChangesOptions.Batch, new AsyncCallback(OnSaveComplete), null);
    }

    // Operation Callbacks
    void OnLoadGamesComplete(IAsyncResult result)
    {
      LoadEntityCompleteEventArgs<Game> eventResult = EndQuery<Game>(result);

      if (LoadGamesComplete != null)
      {
        // Marhall to UI Thread
        Deployment.Current.Dispatcher.BeginInvoke(() =>
          {
            LoadGamesComplete(this, eventResult);
          });
      }
    }

    void OnGenresComplete(IAsyncResult result)
    {
      LoadEntityCompleteEventArgs<Genre> eventResult = EndQuery<Genre>(result);

      if (LoadGenresComplete != null)
      {
        // Marhall to UI Thread
        Deployment.Current.Dispatcher.BeginInvoke(() =>
        {
          LoadGenresComplete(this, eventResult);
        });
      }
    }

    void OnRatingsComplete(IAsyncResult result)
    {
      LoadEntityCompleteEventArgs<Rating> eventResult = EndQuery<Rating>(result);

      if (LoadRatingsComplete != null)
      {
        // Marhall to UI Thread
        Deployment.Current.Dispatcher.BeginInvoke(() =>
        {
          LoadRatingsComplete(this, eventResult);
        });
      }
    }

    void OnSaveComplete(IAsyncResult result)
    {
      DataServiceResponse response = ctx.EndSaveChanges(result);
      OperationResponse[] errors = response.Where(r => r.Error != null).ToArray();

      ErrorEventArgs args = null;
      if (errors.Length > 0)
      {
        theLogger.Log(string.Concat("Error during Save: ", errors[0].Error.Message), Category.Exception, Priority.None);
        args = new ErrorEventArgs(errors[0].Error);
      }
      else
      {
        theLogger.Log("Saved Data", Category.Debug, Priority.None);
        args = new ErrorEventArgs(); ;
      }

      if (SaveComplete != null) SaveComplete(this, args);
    }

    // Used to end a query and capture the error if any
    LoadEntityCompleteEventArgs<T> EndQuery<T>(IAsyncResult result)
    {
      var qry = (DataServiceQuery<T>)result.AsyncState;
      LoadEntityCompleteEventArgs<T> eventResult = null;
      try
      {
        T[] entities = qry.EndExecute(result).ToArray();

        // Log Results
        theLogger.Log(string.Format("{0} Loaded (Count: {1})", typeof(T).Name, entities.Count()), Category.Debug, Priority.None);

        // Wire for Updates to the DataService (to support saves)
        foreach (T ent in entities)
        {
          INotifyPropertyChanged notifier = ent as INotifyPropertyChanged;
          if (notifier != null) notifier.PropertyChanged += new PropertyChangedEventHandler(notifier_PropertyChanged);
        }
        
        // Create the event that returns the data
        eventResult = new LoadEntityCompleteEventArgs<T>(entities);
      }
      catch (Exception ex)
      {
        theLogger.Log(string.Concat(string.Format("Error during Loading of {0}: ", typeof(T).Name), ex.Message), Category.Exception, Priority.None);

        // Throw the event with the error
        eventResult = new LoadEntityCompleteEventArgs<T>(ex);
      }

      return eventResult;
    }

    // Event Handlers
    void notifier_PropertyChanged(object sender, PropertyChangedEventArgs e)
    {
      // Tell the Context the object has changed (for saving)
      ctx.UpdateObject(sender);
    }
  }
}
