﻿using System;
using System.Collections.ObjectModel;
using GameEditor.Client.Data;
using GameEditor.Client.Data.DataServices;
using GameEditor.Client.Data.EventArgs;
using GameEditor.Client.Data.Events;
using GameEditor.Common;
using GameEditor.Common.Interfaces;
using Microsoft.Practices.Composite.Events;
using Microsoft.Practices.Composite.Presentation.Commands;
using Microsoft.Practices.Composite.Presentation.Events;
using System.ComponentModel;

namespace GameEditor.GameDetails.ViewModels
{
  public class GameDetailsViewModel : ViewModelBase, IGameDetailsViewModel
  {
    Game theGame = null;
    IGameModel theModel = null;
    ObservableCollection<Genre> theGenres = new ObservableCollection<Genre>();
    ObservableCollection<Rating> theRatings = new ObservableCollection<Rating>();
    PropertyChangedEventHandler gameWatcher;
    string errorMsg = "";
    bool isGameNull = false;
    bool isSaving = false;

    public GameDetailsViewModel(IEventAggregator aggregator, IGameModel model)
    {
      // Register for the aggregated event
      aggregator.GetEvent<GameSelectedEvent>().Subscribe(SetGame, ThreadOption.UIThread, false);
    
      // Create the DelegateCommand
      SaveCommand = new DelegateCommand<object>(c => Save(), c => CanSave());

      // Wire up the Model
      theModel = model;
      theModel.LoadGenresComplete += new EventHandler<LoadEntityCompleteEventArgs<Genre>>(theModel_LoadGenresComplete);
      theModel.LoadRatingsComplete += new EventHandler<LoadEntityCompleteEventArgs<Rating>>(theModel_LoadRatingsComplete);
      theModel.SaveComplete += new EventHandler<ErrorEventArgs>(theModel_SaveComplete);
      
      // Start Loading the Genres and Ratings in the background
      theModel.LoadGenresAsync();
      theModel.LoadRatingsAsync();

      // Create the watcher delegate
      gameWatcher = new PropertyChangedEventHandler(theGame_PropertyChanged);
    }

    public DelegateCommand<object> SaveCommand { get; protected set; }

    public void SetGame(Game g)
    {
      CurrentGame = g;
    }

    public Game CurrentGame
    {
      get { return theGame; }
      set
      {
        if (theGame != value)
        {
          // Unregister event for old game
          if (theGame != null) theGame.PropertyChanged -= gameWatcher;

          // Change the value
          theGame = value;
          
          // Register for updates
          theGame.PropertyChanged += gameWatcher;

          // Notify that the current game as changed
          FirePropertyChanged("CurrentGame");

          // Mark whether the game is valid
          IsGameSelected = theGame != null ? true : false;

          // Make sure the command knows the new game has changed
          SaveCommand.RaiseCanExecuteChanged();
        }
      }
    }

    void theGame_PropertyChanged(object sender, System.ComponentModel.PropertyChangedEventArgs e)
    {
      // Ensure that as the game's properties change, 
      // the we check to update the SaveCommand
      SaveCommand.RaiseCanExecuteChanged();
    }

    public ObservableCollection<Genre> Genres { get { return theGenres; } }
    public ObservableCollection<Rating> Ratings { get { return theRatings; } }

    public IModel Model { get; set; }

    void theModel_LoadRatingsComplete(object sender, LoadEntityCompleteEventArgs<Rating> e)
    {
      if (e.Error == null)
      {
        Ratings.Clear();
        foreach (var rating in e.Results) Ratings.Add(rating);
      }
      else
      {
        ErrorMessage = string.Concat("Error Occurred: ", e.Error.Message);
      }
    }

    void theModel_LoadGenresComplete(object sender, GameEditor.Client.Data.EventArgs.LoadEntityCompleteEventArgs<Genre> e)
    {
      if (e.Error == null)
      {
        Genres.Clear();
        foreach (var rating in e.Results) Genres.Add(rating);
      }
      else
      {
        ErrorMessage = string.Concat("Error Occurred: ", e.Error.Message);
      }
    }

    public void Save()
    {
      IsSaving = true;
      theModel.SaveAsync();
    }

    public bool CanSave()
    {
      return IsGameSelected ? CurrentGame.IsValid() : false;
    }

    void theModel_SaveComplete(object sender, ErrorEventArgs e)
    {
      if (e.Error != null)
      {
        ErrorMessage = string.Concat("Error occurred while saving: ", e.Error.Message);
      }
      IsSaving = false;
    }

    public string ErrorMessage
    {
      get
      {
        return errorMsg;
      }
      private set
      {
        if (errorMsg != value)
        {
          value = errorMsg;
          FirePropertyChanged("ErrorMessage");
        }
      }
    }

    public bool IsGameSelected
    {
      get { return isGameNull; }
      set
      {
        if (value != isGameNull)
        {
          isGameNull = value;
          FirePropertyChanged("IsGameSelected");
        }
      }
    }

    public bool IsSaving
    {
      get { return isSaving; }
      set
      {
        if (value != isSaving)
        {
          isSaving = value;
          FirePropertyChanged("IsSaving");
        }
      }
    }

  }
}
