﻿using System;
using System.Collections.ObjectModel;
using GameEditor.Client.Data;
using GameEditor.Client.Data.DataServices;
using GameEditor.Client.Data.EventArgs;
using GameEditor.Client.Data.Events;
using GameEditor.Common;
using GameEditor.Common.Interfaces;
using Microsoft.Practices.Composite.Events;
using Microsoft.Practices.Composite.Presentation.Commands;
using Microsoft.Practices.Composite.Modularity;

namespace GameEditor.GameList.ViewModels
{
  public class GameListViewModel : ViewModelBase, IGameListViewModel
  {
    IGameModel theModel = null;
    IEventAggregator theEventAggregator = null;
    IModuleManager theModuleManager = null;
    string error = null;
    bool isLoading = false;

    public GameListViewModel(IGameModel model, IEventAggregator eventAggregator, IModuleManager modMgr)
    {
      // Keep a reference to injected services
      theEventAggregator = eventAggregator;
      theModel = model;
      theModuleManager = modMgr;

      // Initialize the bindable collection
      Games = new ObservableCollection<Game>();

      // Setup the Commands
      LoadCommand = new DelegateCommand<object>((o) => LoadAsync());
      SelectionChanged = new DelegateCommand<object>(o => HandleSelectionChanged(o));

      // Wire up Events
      theModel.LoadGamesComplete += new EventHandler<LoadEntityCompleteEventArgs<Game>>(theModel_LoadGamesComplete);
    }

    private void HandleSelectionChanged(object o)
    {
      // Fire Selection Changed with Global Event
      theEventAggregator.GetEvent<GameSelectedEvent>().Publish(o as Game);
    }

    public void LoadAsync()
    {
      IsLoading = true;
      theModel.LoadGamesAsync();
    }

    public void LoadByGenreAsync(string genre)
    {
      IsLoading = true;
      theModel.LoadGamesByGenreAsync(genre);
    }

    void theModel_LoadGamesComplete(object sender, LoadEntityCompleteEventArgs<Game> e)
    {
      if (e.Error != null)
      {
        LoadError = string.Concat("Load Failed: ", e.Error.Message);
      }
      else
      {
        Games.Clear();

        foreach (var game in e.Results)
        {
          Games.Add(game);
        }

        if (CollectionLoaded != null) CollectionLoaded(this, new EventArgs());
      }

      // Since we now have games, let's load the detail pane
      theModuleManager.LoadModule("GameEditor.GameDetails");

      // Turn off loading
      IsLoading = false;
    }

    public event EventHandler CollectionLoaded;

    public DelegateCommand<object> LoadCommand { get; protected set;}
    public DelegateCommand<object> SelectionChanged { get; protected set; }

    public string LoadError 
    {
      get { return error; } 
      private set
      {
        if (error != value)
        {
          error = value;
          FirePropertyChanged("LoadError");
        }
      } 
    }

    public bool IsLoading
    {
      get { return isLoading; }
      private set
      {
        if (isLoading != value)
        {
          isLoading = value;
          FirePropertyChanged("IsLoading");
        }
      }
    }

    public ObservableCollection<Game> Games { get; protected set; }

    public IModel Model
    {
      get
      {
        return theModel;
      }
      set
      {
        theModel = value as IGameModel;
      }
    }
  }
}
