﻿Imports System
Imports System.ComponentModel
Imports System.Data.Services.Client
Imports System.Linq
Imports System.Threading
Imports GameEditor.Client.Data.DataServices
Imports GameEditor.Client.Data.EventArgs
Imports GameEditor.Common
Imports Microsoft.Practices.Composite.Logging
Imports System.Windows

Public Class GameModel
  Implements IGameModel
  ' Fields
  Private ctx As New GameEntities(New Uri("GameService.svc", UriKind.Relative))
  Private theLogger As ILoggerFacade

  ' Construction
  Public Sub New(ByVal logger As ILoggerFacade)
    theLogger = logger
  End Sub

  ' Events
  Public Event LoadGamesComplete As  _
    EventHandler(Of LoadEntityCompleteEventArgs(Of Game)) _
    Implements IGameModel.LoadGamesComplete
  Public Event LoadGenresComplete As  _
    EventHandler(Of LoadEntityCompleteEventArgs(Of Genre)) _
    Implements IGameModel.LoadGenresComplete
  Public Event LoadRatingsComplete As  _
    EventHandler(Of LoadEntityCompleteEventArgs(Of Rating)) _
    Implements IGameModel.LoadRatingsComplete
  Public Event SaveComplete As EventHandler(Of ErrorEventArgs) _
    Implements IGameModel.SaveComplete

  ' Operations
  Public Sub LoadGamesAsync() Implements IGameModel.LoadGamesAsync
    Dim qry = TryCast((From g In ctx.Games _
                       Order By g.ReleaseDate Descending _
                       Select g), DataServiceQuery(Of Game))

    qry.BeginExecute(New AsyncCallback(AddressOf OnLoadGamesComplete), qry)
  End Sub

  Public Sub LoadGamesByGenreAsync(ByVal genre As String) Implements IGameModel.LoadGamesByGenreAsync
    Dim qry = TryCast((From g In ctx.Games _
                       Where g.Genre.Name = genre _
                       Order By g.ReleaseDate Descending _
                       Select g), DataServiceQuery(Of Game))

    qry.BeginExecute(New AsyncCallback(AddressOf OnLoadGamesComplete), qry)
  End Sub

  Public Sub LoadGenresAsync() Implements IGameModel.LoadGenresAsync
    Dim qry = TryCast((From g In ctx.Genres _
                       Order By g.Name _
                       Select g), DataServiceQuery(Of Genre))

    qry.BeginExecute(New AsyncCallback(AddressOf OnGenresComplete), qry)
  End Sub

  Public Sub LoadRatingsAsync() Implements IGameModel.LoadRatingsAsync
    Dim qry = TryCast((From g In ctx.Ratings _
                       Order By g.Name _
                       Select g), DataServiceQuery(Of Rating))

    qry.BeginExecute(New AsyncCallback(AddressOf OnRatingsComplete), qry)
  End Sub

  ' NOTE: This saves everything that's changed in the Model, 
  '       not just the currently edited item. 
  '       Could change this behavior and support undo and such
  '       but not relevant to the way that Prism works
  Public Sub SaveAsync() Implements IGameModel.SaveAsync
    ctx.BeginSaveChanges(SaveChangesOptions.Batch, New AsyncCallback(AddressOf OnSaveComplete), Nothing)
  End Sub

  ' Operation Callbacks
  Private Sub OnLoadGamesComplete(ByVal result As IAsyncResult)
    Dim eventResult As LoadEntityCompleteEventArgs(Of Game) = EndQuery(Of Game)(result)

    ' Marhall to UI Thread
    Deployment.Current.Dispatcher.BeginInvoke( _
      CType(AddressOf FireLoadGamesComplete,  _
            Action(Of LoadEntityCompleteEventArgs(Of Game))), _
            eventResult)

  End Sub

  Private Sub FireLoadGamesComplete(ByVal eventResult As LoadEntityCompleteEventArgs(Of Game))
    RaiseEvent LoadGamesComplete(Me, eventResult)
  End Sub

  Private Sub OnGenresComplete(ByVal result As IAsyncResult)
    Dim eventResult As LoadEntityCompleteEventArgs(Of Genre) = EndQuery(Of Genre)(result)

    ' Marhall to UI Thread
    Deployment.Current.Dispatcher.BeginInvoke( _
      CType(AddressOf FireLoadGenresComplete,  _
            Action(Of LoadEntityCompleteEventArgs(Of Genre))), _
            eventResult)
  End Sub

  Private Sub FireLoadGenresComplete(ByVal eventResult As LoadEntityCompleteEventArgs(Of Genre))
    RaiseEvent LoadGenresComplete(Me, eventResult)
  End Sub

  Private Sub OnRatingsComplete(ByVal result As IAsyncResult)
    Dim eventResult As LoadEntityCompleteEventArgs(Of Rating) = EndQuery(Of Rating)(result)

    ' Marhall to UI Thread
    Deployment.Current.Dispatcher.BeginInvoke( _
      CType(AddressOf FireLoadRatingsComplete,  _
            Action(Of LoadEntityCompleteEventArgs(Of Rating))), _
            eventResult)
  End Sub

  Private Sub FireLoadRatingsComplete(ByVal eventResult As LoadEntityCompleteEventArgs(Of Rating))
    RaiseEvent LoadRatingsComplete(Me, eventResult)
  End Sub

  Private Sub OnSaveComplete(ByVal result As IAsyncResult)
    Dim response As DataServiceResponse = ctx.EndSaveChanges(result)
    Dim errors() As OperationResponse = response.Where(Function(r) r.Error IsNot Nothing).ToArray()

    Dim args As ErrorEventArgs = Nothing
    If errors.Length > 0 Then
      theLogger.Log(String.Concat("Error during Save: ", errors(0).Error.Message), Category.Exception, Priority.None)
      args = New ErrorEventArgs(errors(0).Error)
    Else
      theLogger.Log("Saved Data", Category.Debug, Priority.None)
      args = New ErrorEventArgs()

    End If

    ' Marhall to UI Thread
    Deployment.Current.Dispatcher.BeginInvoke( _
      CType(AddressOf FireSaveComplete,  _
            Action(Of ErrorEventArgs)), _
            args)
  End Sub

  Private Sub FireSaveComplete(ByVal eventResult As ErrorEventArgs)
    RaiseEvent SaveComplete(Me, eventResult)
  End Sub


  ' Used to end a query and capture the error if any
  Private Function EndQuery(Of T)(ByVal result As IAsyncResult) As LoadEntityCompleteEventArgs(Of T)
    Dim qry = CType(result.AsyncState, DataServiceQuery(Of T))
    Dim eventResult As LoadEntityCompleteEventArgs(Of T) = Nothing
    Try
      Dim entities() As T = qry.EndExecute(result).ToArray()

      ' Log Results
      theLogger.Log(String.Format("{0} Loaded (Count: {1})", GetType(T).Name, entities.Count()), Category.Debug, Priority.None)

      ' Wire for Updates to the DataService (to support saves)
      For Each ent As T In entities
        Dim notifier As INotifyPropertyChanged = TryCast(ent, INotifyPropertyChanged)
        If notifier IsNot Nothing Then
          AddHandler notifier.PropertyChanged, AddressOf notifier_PropertyChanged
        End If
      Next ent

      ' Create the event that returns the data
      eventResult = New LoadEntityCompleteEventArgs(Of T)(entities)
    Catch ex As Exception
      theLogger.Log(String.Concat(String.Format("Error during Loading of {0}: ", GetType(T).Name), ex.Message), Category.Exception, Priority.None)

      ' Throw the event with the error
      eventResult = New LoadEntityCompleteEventArgs(Of T)(ex)
    End Try

    Return eventResult
  End Function

  ' Event Handlers
  Private Sub notifier_PropertyChanged(ByVal sender As Object, ByVal e As PropertyChangedEventArgs)
    ' Tell the Context the object has changed (for saving)
    ctx.UpdateObject(sender)
  End Sub

End Class