﻿Imports System
Imports System.Collections.ObjectModel
Imports GameEditor.Client.Data
Imports GameEditor.Client.Data.DataServices
Imports GameEditor.Client.Data.EventArgs
Imports GameEditor.Client.Data.Events
Imports GameEditor.Common
Imports GameEditor.Common.Interfaces
Imports Microsoft.Practices.Composite.Events
Imports Microsoft.Practices.Composite.Presentation.Commands
Imports Microsoft.Practices.Composite.Presentation.Events
Imports System.ComponentModel

Namespace ViewModels
  Public Class GameDetailsViewModel
    Inherits ViewModelBase
    Implements IGameDetailsViewModel

    Private theGame As Game = Nothing
    Private theModel As IGameModel = Nothing
    Private theGenres As ObservableCollection(Of Genre) = New ObservableCollection(Of Genre)()
    Private theRatings As ObservableCollection(Of Rating) = New ObservableCollection(Of Rating)()
    Private gameWatcher As PropertyChangedEventHandler
    Private errorMsg As String = ""
    Private isGameNull As Boolean = False
    Private isSaveHappening As Boolean = False

    Public Sub New(ByVal aggregator As IEventAggregator, ByVal model As IGameModel)
      aggregator.GetEvent(Of GameSelectedEvent)().Subscribe(AddressOf SetGame, ThreadOption.UIThread, False)

      ' Create the DelegateCommand
      SaveCommand = New DelegateCommand(Of Object)(AddressOf Save, AddressOf CanSave)

      ' Wire up the Model
      theModel = model
      AddHandler theModel.LoadGenresComplete, AddressOf theModel_LoadGenresComplete
      AddHandler theModel.LoadRatingsComplete, AddressOf theModel_LoadRatingsComplete
      AddHandler theModel.SaveComplete, AddressOf theModel_SaveComplete

      ' Start Loading the Genres and Ratings in the background
      theModel.LoadGenresAsync()
      theModel.LoadRatingsAsync()

      ' Create the watcher delegate
      gameWatcher = New PropertyChangedEventHandler(AddressOf theGame_PropertyChanged)

    End Sub

    Private Sub theGame_PropertyChanged(ByVal sender As Object, ByVal e As PropertyChangedEventArgs)

      ' Ensure that as the game's properties change, 
      ' the we check to update the SaveCommand
      SaveCommand.RaiseCanExecuteChanged()

    End Sub

    Private privateSaveCommand As DelegateCommand(Of Object)
    Public Property SaveCommand() As DelegateCommand(Of Object)
      Get
        Return privateSaveCommand
      End Get
      Protected Set(ByVal value As DelegateCommand(Of Object))
        privateSaveCommand = value
      End Set
    End Property

    Public Sub SetGame(ByVal g As Game)
      CurrentGame = g 
    End Sub

    Public Property CurrentGame() As Game Implements IGameDetailsViewModel.CurrentGame
      Get
        Return theGame
      End Get
      Set(ByVal value As Game)
        If theGame IsNot value Then
          ' Unregister event for old game
          If theGame IsNot Nothing Then
            RemoveHandler theGame.PropertyChanged, gameWatcher
          End If

          ' Change the value
          theGame = value

          ' Register for updates
          AddHandler theGame.PropertyChanged, gameWatcher

          ' Notify that the current game as changed
          FirePropertyChanged("CurrentGame")

          ' Mark whether the game is valid
          IsGameSelected = If(theGame IsNot Nothing, True, False)

          ' Make sure the command knows the new game has changed
          SaveCommand.RaiseCanExecuteChanged()
        End If
      End Set
    End Property

    Public ReadOnly Property Genres() As ObservableCollection(Of Genre) _
      Implements IGameDetailsViewModel.Genres

      Get
        Return theGenres
      End Get
    End Property

    Public ReadOnly Property Ratings() As ObservableCollection(Of Rating) _
      Implements IGameDetailsViewModel.Ratings

      Get
        Return theRatings
      End Get
    End Property

    Private privateModel As IModel

    Public Property Model() As IModel Implements IGameDetailsViewModel.Model
      Get
        Return privateModel
      End Get
      Set(ByVal value As IModel)
        privateModel = value
      End Set
    End Property

    Private Sub theModel_LoadRatingsComplete(ByVal sender As Object, ByVal e As LoadEntityCompleteEventArgs(Of Rating))
      If e.Error Is Nothing Then
        Ratings.Clear()
        For Each rating In e.Results
          Ratings.Add(rating)
        Next rating
      Else
        ErrorMessage = String.Concat("Error Occurred: ", e.Error.Message)
      End If
    End Sub

    Private Sub theModel_LoadGenresComplete(ByVal sender As Object, ByVal e As GameEditor.Client.Data.EventArgs.LoadEntityCompleteEventArgs(Of Genre))
      If e.Error Is Nothing Then
        Genres.Clear()
        For Each rating In e.Results
          Genres.Add(rating)
        Next rating
      Else
        ErrorMessage = String.Concat("Error Occurred: ", e.Error.Message)
      End If
    End Sub

    Public Sub Save() Implements IGameDetailsViewModel.Save
      IsSaving = True
      theModel.SaveAsync()
    End Sub

    Public Function CanSave() As Boolean
      Return If(IsGameSelected, CurrentGame.IsValid(), False)
    End Function

    Private Sub theModel_SaveComplete(ByVal sender As Object, ByVal e As ErrorEventArgs)
      If e.Error IsNot Nothing Then
        ErrorMessage = String.Concat("Error occurred while saving: ", e.Error.Message)
      End If
      IsSaving = False
    End Sub

    Public Property ErrorMessage() As String
      Get
        Return errorMsg
      End Get
      Private Set(ByVal value As String)
        If errorMsg <> value Then
          value = errorMsg
          FirePropertyChanged("ErrorMessage")
        End If
      End Set
    End Property

    Public Property IsGameSelected() As Boolean
      Get
        Return isGameNull
      End Get
      Set(ByVal value As Boolean)
        If value <> isGameNull Then
          isGameNull = value
          FirePropertyChanged("IsGameSelected")
        End If
      End Set
    End Property

    Public Property IsSaving() As Boolean
      Get
        Return isSaveHappening
      End Get
      Set(ByVal value As Boolean)
        If value <> isSaveHappening Then
          isSaveHappening = value
          FirePropertyChanged("IsSaving")
        End If
      End Set
    End Property

  End Class
End Namespace